@@ -1,5 +1,6 @@
 /*
  *  Copyright (C) 2001,2002,2003 Nikos Mavroyanopoulos
+ *  Copyright (C) 2004 Free Software Foundation
  *
  *  This file is part of GNUTLS.
  *
@@ -94,33 +95,15 @@ int _gnutls_pkcs1_rsa_encrypt(gnutls_datum * ciphertext,
 			gnutls_afree(edata);
 			return ret;
 		}
-		for (i = 0; i < psize; i++) {
-			opaque rnd[3];
-
-			/* Read three random bytes that will be
-			 * used to replace the zeros.
-			 */
-			if ( (ret=_gnutls_get_random( rnd, 3, GNUTLS_STRONG_RANDOM)) < 0) {
-				gnutls_assert();
-				gnutls_afree(edata);
-				return ret;
-			}
-			/* use non zero values for 
-			 * the first two.
-			 */
-			if (rnd[0]==0) rnd[0] = 0xaf;
-			if (rnd[1]==0) rnd[1] = 0xae;
-
-			if (ps[i] == 0) {
-				/* If the first one is zero then set it to rnd[0].
-				 * If the second one is zero then set it to rnd[1].
-				 * Otherwise add (mod 256) the two previous ones plus rnd[3], or use
-				 * rnd[1] if the value == 0.
-				 */
-				if (i<2) ps[i] = rnd[i];
-				else ps[i] = GMAX( rnd[3] + ps[i-1] + ps[i-2], rnd[1]);
-			}
-		}
+		for (i = 0; i < psize; i++)
+		  while (ps[i] == 0) {
+		    if ((ret =
+			 _gnutls_get_random(&ps[i], 1, GNUTLS_STRONG_RANDOM)) < 0) {
+		      gnutls_assert();
+		      gnutls_afree(edata);
+		      return ret;
+		    }
+		  }
 		break;
 	case 1:
 		/* using private key */
@@ -491,14 +474,15 @@ int _gnutls_dsa_verify( const gnutls_datum* vdata, const gnutls_datum *sig_value
 	}
 
 	/* decrypt signature */
-	if ( (ret=_gnutls_pk_verify( GCRY_PK_DSA, mdata, rs, params, 
-		params_len)) < 0) {
-	    _gnutls_mpi_release(&mdata);
-	     gnutls_assert();
-	     return ret;
-	}
+	ret=_gnutls_pk_verify( GCRY_PK_DSA, mdata, rs, params, 
+		params_len);
 	_gnutls_mpi_release(&mdata);
 
+	if (ret < 0) {
+		gnutls_assert();
+		return ret;
+	}
+
 	return 0; /* ok */
 }
 
